@@ -28,5 +28,7 @@
 from .protocol_subtomo_MRA import DynamoSubTomoMRA
 from .protocol_extraction import DynamoExtraction
 from .protocol_boxing import DynamoBoxing
-from .protocol_models import DynamoModels
 from .protocol_import_subtomos import DynamoImportSubtomos
+from .protocol_subBoxing import DynamoSubBoxing
+from .protocol_model_workflow import DynamoModelWorkflow
+from .protocol_import_tomograms import DynamoImportTomograms
